/**
* DS3231.h
* Leeman Geophysical LLC (c) 2018
* DS3231 RTC
* 
* DS3231 RTC module headers.
*/

#ifndef __DS3231_H__
#define __DS3231_H__

#define TEMP_OUT_L 0x05
#define TEMP_OUT_H 0x06
#define REG_SECONDS 0x00
#define REG_MINUTES 0x01
#define REG_HOURS 0x02
#define REG_DAY 0x03
#define REG_DATE 0x04
#define REG_MONTH_CENTURY 0x05
#define REG_YEARS 0x06
#define REG_ALARM_1_SECONDS 0x07
#define REG_ALARM_1_MINUTES 0x08
#define REG_ALARM_1_HOURS 0x09
#define REG_ALARM_1_DAY_DATE 0x0A
#define REG_ALARM_2_MINUTES 0x0B
#define REG_ALARM_2_HOURS 0x0C
#define REG_ALARM_2_DAY_DATE 0x0D
#define REG_CONTROL 0x0E
#define REG_STATUS 0x0F
#define REG_AGING_OFFSET 0x10
#define REG_TEMP_MSB 0x11
#define REG_TEMP_LSB 0x12

class DS3231 {
  public:
    void start(int address, char data_pin, char clk_pin, char sq_pin);
    void ocsillator_enable(char enable);
    void square_wave_enable(char enable);
    void convert_temperature_enable(char enable);
    //void convert_temperature;
    void square_wave_frequency(char rate);
    void interrupt_control(char enable);
    void alarm_two_enable(char enable);
    void alarm_one_enable(char enable);
    char get_oscillator_stop_flag(void);
    void clear_oscillator_stop_flag(void);
    void clock_output_enable(char enable);
    char get_busy(void);
    char get_alarm_one_flag(void);
    char get_alarm_two_flag(void);
    void set_aging_offset(char offset);
    int get_temperature(void);
    void set_time(char year, char month, char day, char hour, char minute, char second);
    void read_time(void);
    void change_byte_register(int reg, char mask, char value);
    char seconds;
    char minutes;
    char hours;
    char date;
    char month;
    char year;
    int valid_cnt;
  
  private:
    int dev_address;
    char square_pin;
    char i2c_bus;
};  

#endif
