/**
* TML.h
* Leeman Geophysical LLC (c) 2018
* Tilt Meter Logger
* 
* Main state machine of the tilt meter logger.
*/

#ifndef __TML_H__
#define __TML_H__

#include "datetime.h"
#include "ptm100.h"
#include "status.h"
#include <fdserial.h>
#include "cbuffer.h"
#include "controlpanel.h"
#include "DS3231.h"


// Objects
datetime rtcdatetime;
ptm100 tiltmeter;
status sysstatus;
DS3231 rtc;
fdserial *comp_serial;

// Globals
char sd_mounted = 0;

// Main circular data buffer
cbuffer data_ring;

// Helpers
int get_milliseconds(void);
void rtc_runner(void *par);
void receive_runner(void *par);
void send_debug(char *msg);
void get_filename(char *buffer, datetime dt);
void echo_packet(void);

// State Machine Handlers
void handle_state_initialize(void);
void handle_state_update_panel(void);
void handle_state_write_data(void);
void handle_state_near_time_check(void);
void handle_state_get_n_readings(void);

// Cogs
int rtc_cog; 
int rtcstack[150];
int rx_cog;
int rxstack[150];

volatile int do_rtc_set = 0;
volatile datetime rtcsettime;

#endif
