#include "pins.h"
#include "simpletools.h"
#include "controlpanel.h"
#include "status.h"
#include "DS3231.h"
#include "datetime.h"
#include <fdserial.h>

extern status sysstatus;
extern fdserial *comp_serial;
extern char do_rtc_set;
extern datetime rtcsettime;

void front_panel::handle_serial(void){
  /*
   * Read any serial characters from the computer and process to set system time.
   */
 
  char rx_byte;
  
  while(fdserial_rxReady(comp_serial)) {  // Loop while data are in the buffer
    rx_byte = fdserial_rxChar(comp_serial);  // Get the byte in the buffer
    // Add byte to buffer and move position indicator
    panel_ser_buffer[panel_ser_buffer_position] = rx_byte;
    dprint(comp_serial, "%d", panel_ser_buffer_position);
    panel_ser_buffer_position++;    
    
    // If the received character is a \n, let's see if this is a full sentence
    if (rx_byte == 10) {
      // Is the 10th position is T, let's try to parse
      if (panel_ser_buffer[10] == 84) {
        sscanf(panel_ser_buffer, "%04d/%02d/%02dT%02d:%02d:%02d", &rtcsettime.year, &rtcsettime.month, &rtcsettime.date, &rtcsettime.hour, &rtcsettime.minute, &rtcsettime.second);
        //dprint(comp_serial, "Time Set: %d/%d/%d %d:%d:%d\n", rtcsettime.year, rtcsettime.month, rtcsettime.date, rtcsettime.hour, rtcsettime.minute, rtcsettime.second);
        do_rtc_set = 1;     
      }
    // Clear out the buffer and reset the pointer
    panel_ser_buffer_position = 0;  // Reset buffer position
    memset(panel_ser_buffer, 0, sizeof panel_ser_buffer);  // Clear out buffer
    }       
  }     
}  


void front_panel::disable_panel_lights() {
  /*
   * Turn off all the front panel lights by making them inputs.
   */
  set_direction(PIN_READY_LED, 0);
  set_direction(PIN_ERROR_LED, 0);
  set_direction(PIN_LOGGING_LED, 0);
  set_direction(PIN_WRITING_LED, 0);
  set_direction(PIN_GPS_LED, 0);
}  

void front_panel::enable_panel_lights() {
  /*
   * Enable the front panel by making the LED pins outputs.
   */
  set_direction(PIN_READY_LED, 1);
  set_direction(PIN_ERROR_LED, 1);
  set_direction(PIN_LOGGING_LED, 1);
  set_direction(PIN_WRITING_LED, 1);
  set_direction(PIN_GPS_LED, 1);
}


void front_panel::update_lights() {
  /*
   * Update the status of the front panel lights.
   */
  
  // If the panel lights are set to off, make sure they are disabled and return
  if (light_status == 0) {
    disable_panel_lights();
    return;
  }
  
  // If the panel lights are on, update them
  enable_panel_lights();
  set_output(PIN_LOGGING_LED, logging_status);  // Logging LED is easy
  
  // Ready LED - on if there are no errors and there is an SD card present
  if (sd_present_status) {  //TODO Add error conditions here
    high(PIN_READY_LED);
  }    
  else {
    low(PIN_READY_LED);
  }    
  
  // Error LED - on if the error flag is present
  if (error_status) {
    high(PIN_ERROR_LED);
    //dprint(comp_serial, "ERROR: %d\n", sysstatus._status_reg);
  }
  else {
    low(PIN_ERROR_LED);
  }        
  
  // Writing LED is manipulated by the SD card code - no touching!  
}
    

void front_panel::update_switches() {
  short mux_values = 0;
  // Read ch0 on mux
  low(PIN_MUX_SEL_C);
  low(PIN_MUX_SEL_B);
  low(PIN_MUX_SEL_A);
  mux_values = mux_values | input(PIN_MUX1_DATA);
  mux_values = mux_values << 1;
  mux_values = mux_values | input(PIN_MUX2_DATA);
  mux_values = mux_values << 1;
  
  // Read ch1 on mux
  low(PIN_MUX_SEL_C);
  low(PIN_MUX_SEL_B);
  high(PIN_MUX_SEL_A);
  mux_values = mux_values | input(PIN_MUX1_DATA);
  mux_values = mux_values << 1;
  mux_values = mux_values | input(PIN_MUX2_DATA);
  mux_values = mux_values << 1;

  // Read ch2 on mux
  low(PIN_MUX_SEL_C);
  high(PIN_MUX_SEL_B);
  low(PIN_MUX_SEL_A);
  mux_values = mux_values | input(PIN_MUX1_DATA);
  mux_values = mux_values << 1;
  mux_values = mux_values | input(PIN_MUX2_DATA);
  mux_values = mux_values << 1;

  // Read ch3 on mux
  low(PIN_MUX_SEL_C);
  high(PIN_MUX_SEL_B);
  high(PIN_MUX_SEL_A);
  mux_values = mux_values | input(PIN_MUX1_DATA);
  mux_values = mux_values << 1;
  mux_values = mux_values | input(PIN_MUX2_DATA);
  mux_values = mux_values << 1;
  
  // Read ch4 on mux
  high(PIN_MUX_SEL_C);
  low(PIN_MUX_SEL_B);
  low(PIN_MUX_SEL_A);
  mux_values = mux_values | input(PIN_MUX1_DATA);
  mux_values = mux_values << 1;
  mux_values = mux_values | input(PIN_MUX2_DATA);
  mux_values = mux_values << 1;
  
  // Read ch5 on mux
  high(PIN_MUX_SEL_C);
  low(PIN_MUX_SEL_B);
  high(PIN_MUX_SEL_A);
  mux_values = mux_values | input(PIN_MUX1_DATA);
  mux_values = mux_values << 1;
  mux_values = mux_values | input(PIN_MUX2_DATA);
  mux_values = mux_values << 1;
  
  // Read ch6 on mux
  high(PIN_MUX_SEL_C);
  high(PIN_MUX_SEL_B);
  low(PIN_MUX_SEL_A);
  mux_values = mux_values | input(PIN_MUX1_DATA);
  mux_values = mux_values << 1;
  mux_values = mux_values | input(PIN_MUX2_DATA);
  mux_values = mux_values << 1;
  
  // Read ch7 on mux
  high(PIN_MUX_SEL_C);
  high(PIN_MUX_SEL_B);
  high(PIN_MUX_SEL_A);
  mux_values = mux_values | input(PIN_MUX1_DATA);
  mux_values = mux_values << 1;
  mux_values = mux_values | input(PIN_MUX2_DATA);
  
  // Check if the SD card is present and update
  sd_present_status = !input(PIN_SD_DETECT);
  
  // Update status of panel light switch
  switch (mux_values & 0b0000000000010100){
    case 4:
      light_status = 0;
      sysstatus.set_status(2, 0);
      break;
    case 16:
      light_status = 1;
      sysstatus.set_status(2, 0);
      break;
    default:
      light_status = 1;
      sysstatus.set_status(2, 1);
      break;
  }
  
  // Update status of logging switch      
  switch (mux_values & 0b0000000101000000){
    case 64:
      logging_status = 0;
      sysstatus.set_status(3, 0);
      break;
    case 256:
      logging_status = 1;
      sysstatus.set_status(3, 0);
      break;
    default:
      logging_status = 1;
      sysstatus.set_status(3, 1);
      break;
  }    
  
  // Update status of interval switch      
  switch (mux_values & 0b1010101010100000){
    case 10912:
      interval_status = 0;
      sysstatus.set_status(4, 0);
      break;
    case 35488:
      interval_status = 1;
      sysstatus.set_status(4, 0);
      break;
    case 41632:
      interval_status = 5;
      sysstatus.set_status(4, 0);
      break;
    case 43168:
      interval_status = 15;
      sysstatus.set_status(4, 0);
      break;      
    case 43552:
      interval_status = 30;
      sysstatus.set_status(4, 0);
      break;
    case 43648:
      interval_status = 60;
      sysstatus.set_status(4, 0);
      break;
    default:
      interval_status = 0;
      sysstatus.set_status(4, 1);
      break;
  }
  
  
  // Update status of time mode switch      
  switch (mux_values & 0b0101010000000000){
    case 5120:
      time_status = 0;
      sysstatus.set_status(5, 0);
      break;
    case 17408:
      time_status = 1;
      sysstatus.set_status(5, 0);
      break;
    case 20480:
      time_status = 2;
      sysstatus.set_status(5, 0);
      break;
    default:
      time_status = 0;
      sysstatus.set_status(5, 1);
      break;
  }     
}  
