/**
* ptm100.h
* Leeman Geophysical LLC (c) 2018
* Tilt Meter Logger
* 
* Tilt meter interface functionality.
*/

#ifndef __PTM100_H__
#define __PTM100_H__

#define PTM_SER_BUFFER_LEN 180

#include "datetime.h"

struct ptmpacket {
  /*
   * Contains data from a single packet.
   */
  datetime packet_dt;
  int data_valid_cnt;
  int x_tilt_high;
  int x_tilt_low;
  int y_tilt_high;
  int y_tilt_low;
  int x_tilt;
  int y_tilt;
  short board_temperature;
  short mag_temperature;
  short metal_temperature;
  short mag_x;
  short mag_y;
  short mag_z;
  short acc_x;
  short acc_y;
  short acc_z;   
};    


class ptm100 {
  public:
    
    // State information
    char power_state;
  
    // Serial packet information
    char inst_ser_buffer [PTM_SER_BUFFER_LEN];
    char inst_ser_buffer_position;
    char tabs_detected;
  
    void on(void);
    void off(void);
    void read_new_data(void);
    void clear_buffer(void);
    
    // default constructor
    ptm100() {
      power_state = 0;
      inst_ser_buffer_position = 0;
      tabs_detected = 0;
    }  
};  


#endif