/*
* status.cpp
* Leeman Geophysical LLC (c) 2018
* Tilt Meter Logger
* 
* Track status and error conditions of hardware.
*/


#include "status.h"

/*
 * Status/Error Codes (bit set to 1)
 * 0 - SD Mount Error
 * 1 - File buffer write length error (logger.cpp)
 * 2 - Light switch default fallback (controlpanel.cpp)
 * 3 - Logging switch default fallback (controlpanel.cpp)
 * 4 - Interval switch default fallback (controlpanel.cpp)
 * 5 - Time mode switch default fallback (controlpanel.cpp)
 * 6 - 
 * 7 -
 * 8 -
 * 9 -
 * 10 -
 * 11 -
 * 12 -
 * 13 -
 * 14 -
 * 15 -
 * 16 -
 * 17 -
 * 18 -
 * 19 - 
 * 20 - 
 * 21 - 
 * 22 -
 * 23 -
 * 24 -
 * 25 -
 * 26 -
 * 27 -
 * 28 -
 * 29 - 
 * 30 - 
 * 31 - 
 */


void status::clear(void) {
  /*
   * Clear all status bits to the not set/no error state.
   */
  _status_reg = 0;
}


char status::is_error(void) {
  /*
   * Check if any status is in the set/error state.
   * Return 1 if any status is non zero, otherwise return 0.
   */
  if (!_status_reg) {return 0;}
  else {return 1;}  
}


void status::set_status(char snum, int state) {
  /*
   * Set the status of a particular bit in the status register.
   */
  int mask = 1 << snum;
  _status_reg = _status_reg & ~mask; // Clear the bit we are dealing with
  _status_reg = _status_reg | (state << snum);
}

char status::get_status(char snum) {
  /*
   * Get the status of a particular bit in the status register.
   */
  int mask = 1 << snum;
  return (_status_reg & mask) >> snum;
}
 